import requests
from .student import Student

api_url = "https://ske19-api.herokuapp.com/"


class SKE19:
    """A client to access SKE19 API, must be initialized with a secret key.
    """
    def __init__(self, secret: str, duration: int = 3600):
        """Create instance to access SKE19 API, must have secret key provided.

        :param secret: Secret key for generating auth token.
        :type secret: str
        :param duration: Duration for session.
        :type duration: int
        :raises: ValueError: Secret key is incorrect.
        """
        res = requests.post(f"{api_url}auth", headers={"secret": secret,
                            "duration": str(duration)})
        if res.status_code == 403:
            raise ValueError('Cannot authorize the SKE19 client.',
                             f"Secret: {secret}")
        self.__token = res.json()["token"]

    def get_student(self, id: str):
        """Get student information from student ID.

        :param id: Student ID
        :type id: str
        :return: Student object associated with the ID.
        :rtype: Student
        """
        body = {
            "id": id
        }
        headers = {
            "Authorization": f"Bearer {self.__token}",
            "Content-type": "application/json"
        }
        _res = requests.get(f"{api_url}student", json=body, headers=headers)
        res = _res.json()
        if _res.status_code == 404:
            return res["message"]
        return Student(id,
                       res["firstNameEN"], res["lastNameEN"], res["nickEN"],
                       res["firstNameTH"], res["lastNameTH"], res["nickTH"],
                       res["email"],
                       res["instagram"])

    def get_all_students(self):

        headers = {
            "Authorization": f"Bearer {self.__token}",
            "Content-type": "application/json"
        }
        _res = requests.get(f"{api_url}students", headers=headers)
        res = _res.json()["students"]
        students = []
        for std in res:
            students.append(
                Student(std,
                        res[std]["firstNameEN"],
                        res[std]["lastNameEN"],
                        res[std]["nickEN"],
                        res[std]["firstNameTH"],
                        res[std]["lastNameTH"],
                        res[std]["nickTH"],
                        res[std]["email"],
                        res[std]["instagram"],)
            )
        return students



class Student:
    """A class for storing information of a student.
    """
    def __init__(self, id, first_en, last_en, nick_en,
                 first_th, last_th, nick_th, email, ig):
        self._id = id
        self._first_en = first_en
        self._last_en = last_en
        self._nick_en = nick_en
        self._first_th = first_th
        self._last_th = last_th
        self._nick_th = nick_th
        self._email = email
        self._ig = ig

    def get_id(self):
        """Gets an ID of the student.

        :return: ID of student
        :rtype: str
        """
        return self._id

    def get_name_english(self):
        """Gets the first, last and nick name in English of the student.

        :return: Dictionary of first, last and nick name.
        :rtype: dict
        """
        return {
            "first_name": self._first_en,
            "last_name": self._last_en,
            "nick_name": self._nick_en
        }

    def get_name_thai(self):
        """Gets the first, last and nick name in Thai of the student.

        :return: Dictionary of first, last and nick name.
        :rtype: dict
        """
        return {
            "first_name": self._first_th,
            "last_name": self._last_th,
            "nick_name": self._nick_th
        }

    def get_email(self):
        """Gets the email of the student.

        :return: Email string
        :rtype: str
        """
        return self._email

    def get_instagram(self):
        """Gets the instagram ID of the student.

        :return: Instagram ID string
        :rtype: str
        """
        return self._ig

    def __repr__(self):
        return f"Student {self._id}"



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ske19-api",
    version="1.1.1",
    author="SKE19",
    author_email="pawitchaya.ch@ku.th",
    description="A wrapper for SKE19 API.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/SKE19/SKE19-API-Wrappers/tree/main/python3",
    project_urls={
        "Issues": "https://github.com/SKE19/SKE19-API-Wrappers/issues?q=is:issue+is:open+label:python3"
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Education",
        "Topic :: Internet :: WWW/HTTP :: Indexing/Search"
    ],
    package_dir={
        "": "src"
    },
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.7",
    install_requires=[
        "requests"
    ]
)
